#ifndef STA_CORE_BUS_SERIAL_INTERFACE_HPP
#define STA_CORE_BUS_SERIAL_INTERFACE_HPP

#include <sta/mutex.hpp>

#include <cstdint>
#include <cstddef>

namespace sta
{
    /**
     * @brief Abstract interface for serial communication.
    */
    class Interface
    {
    public:
        Interface(Mutex * mutex);

        /**
         * @brief Send single byte of data.
         *
         * @param value     8-bit value
         */
        virtual void transfer(uint8_t value) = 0;
        /**
         * @brief Send two bytes of data.
         *
         * @param value     16-bit value
         */
        virtual void transfer16(uint16_t value) = 0;
        /**
         * @brief Send data from buffer.
         *
         * @param buffer    Source buffer
         * @param size      Number of bytes to transfer
         */
        virtual void transfer(const uint8_t * buffer, size_t size) = 0;
        /**
         * @brief Read incoming data to buffer.
         *
         * @param buffer    Destination buffer
         * @param size      Number of bytes to read
         */
        virtual void receive(uint8_t * buffer, size_t size) = 0;

        /**
         * @brief Send byte value repeatedly.
         *
         * @param value     8-bit value to repeat
         * @param count     Number of repetitions
         */
        virtual void fill(uint8_t value, size_t count) = 0;

        /**
         * @brief Acquire usage rights to use the interface.
         *
         * Must be called before any I/O operations are executed.
         */
        virtual void acquire();
        /**
         * @brief Release usage rights for interface.
         *
         * Must be called after last I/O operation.
         */
        virtual void release();

        /**
         * @returns true if the interface has been aquired.
        */
        bool isAcquired();
    private:
        Mutex * mutex_;
        bool    acquired_ = false;
    };
} // namespace sta


#endif // STA_CORE_BUS_SERIAL_INTERFACE_HPP
